@@ -1,15 +1,15 @@
 # -*- perl -*-
 
 #
-# $Id: XMLViewer.pm,v 1.24 2001/04/29 09:10:30 eserte Exp $
+# $Id: XMLViewer.pm,v 1.29 2003/04/26 23:34:49 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright  2000 Slaven Rezic. All rights reserved.
+# Copyright  2000, 2003 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# Mail: mailto:eserte@cs.tu-berlin.de
-# WWW:  http://www.cs.tu-berlin.de/~eserte/
+# Mail: mailto:slaven@rezic.de
+# WWW:  http://www.rezic.de/eserte/
 #
 
 package Tk::XMLViewer;
@@ -20,12 +20,12 @@ require Tk::Pixmap;
 use strict;
 use vars qw($VERSION);
 
-use base qw(Tk::ROText);
+use base qw(Tk::Derived Tk::ROText);
 use XML::Parser;
 
 Construct Tk::Widget 'XMLViewer';
 
-$VERSION = '0.13';
+$VERSION = '0.15';
 
 my($curr_w); # ugly, but probably faster than defining handlers for everything
 my $indent_width = 32;
@@ -36,23 +36,29 @@ sub SetIndent {
     $indent_width = $arg;
 }
 
-sub InitObject {
+sub Populate {
     my($w,$args) = @_;
-    $w->SUPER::InitObject($args);
+    $w->SUPER::Populate($args);
     $w->configure(-wrap   => 'word',
 		  -cursor => 'left_ptr');
+
+    my $tagcolor     = delete $args->{-tagcolor}     || 'red';
+    my $attrkeycolor = delete $args->{-attrkeycolor} || 'green4';
+    my $attrvalcolor = delete $args->{-attrvalcolor} || 'DarkGreen';
+    my $commentcolor = delete $args->{-commentcolor} || 'gold2';
+
     $w->tagConfigure('xml_tag',
-		     -foreground => 'red',
+		     -foreground => $tagcolor,
 		     #-font => 'boldXXX',
 		     );
     $w->tagConfigure('xml_attrkey',
-		     -foreground => 'green4',
+		     -foreground => $attrkeycolor,
 		     );
     $w->tagConfigure('xml_attrval',
-		     -foreground => 'DarkGreen',
+		     -foreground => $attrvalcolor,
 		     );
     $w->tagConfigure('xml_comment',
-		     -foreground => 'gold2',
+		     -foreground => $commentcolor,
 		     );
     $w->{IndentTags}  = [];
     $w->{RegionCount} = 0;
@@ -68,6 +74,8 @@ sub insertXML {
     $w->Busy();
     my(%args) = @_;
     my $p1 = new XML::Parser(Style => "Stream",
+			     # XXX check!
+			     #XXX nonono! ($Tk::VERSION < 804 ? (ProtocolEncoding => 'UTF-8') : ()),
 			     Handlers => {
   				 Comment => \&hComment,
   				 XMLDecl => \&hDecl,
@@ -78,11 +86,11 @@ sub insertXML {
     $curr_w = $w;
     eval {
 	if ($args{-file}) {
-	    $p1->parsefile($args{-file});
 	    $w->{Source} = ['file', $args{-file}];
+	    $p1->parsefile($args{-file});
 	} elsif (exists $args{-text}) {
-	    $p1->parse($args{-text});
 	    $w->{Source} = ['text', $args{-text}];
+	    $p1->parse($args{-text});
 	} else {
 	    die "-text or -file argument missing";
 	}
@@ -266,16 +274,24 @@ sub ShowHideRegion {
     my(@old_tags) = $w->tagNames("showhidemarkbegin");
     $w->delete("showhidemarkbegin", "showhidemarkend");
     if (!exists $args{-open}) {
-	$args{-open} = $w->tagCget("region" . $region, -elide);
+	if ($Tk::VERSION > 800) {
+	    $args{-open} = $w->tagCget("region" . $region, '-state') eq 'hidden';
+	} else {
+	    $args{-open} = $w->tagCget("region" . $region, '-elide');
+	}
     }
     if ($args{-open}) {
 	$w->imageCreate("showhidemarkbegin",
 			-image => $w->{'MinusImage'});
-	$w->tagConfigure("region" . $region, -elide => undef);
+	$w->tagConfigure("region" . $region,
+			 $Tk::VERSION > 800 ? (-state => '')
+			                    : (-elide => undef));
     } else {
 	$w->imageCreate("showhidemarkbegin",
 			-image => $w->{'PlusImage'});
-	$w->tagConfigure("region" . $region, -elide => 1);
+	$w->tagConfigure("region" . $region,
+			 $Tk::VERSION > 800 ? (-state => 'hidden')
+			                    : (-elide => 1));
     }
     # restore old tags for minus/plus image
     foreach my $tag (@old_tags) {
@@ -399,7 +415,7 @@ warn $tag;
 
 sub XMLMenu {
     my $w = shift;
-    if ($w->can("menu")) {
+    if ($Tk::VERSION > 800.014 && $w->can("menu")) {
 	my $textmenu = $w->menu;
 	my $xmlmenu = $textmenu->cascade(-tearoff => 0,
 					 -label => "XML");
@@ -448,8 +464,8 @@ sub _convert_from_unicode {
 }
 EOF
     } else { # do nothing
+        warn "No unicode decoder found --- consider installing Unicode::String";
         eval <<'EOF';
-require Unicode::String;
 sub _convert_from_unicode { $_[0] }
 EOF
     }
@@ -518,7 +534,41 @@ Tk::XMLViewer is an widget inherited from Tk::Text which displays XML
 in a hierarchical tree. You can use the plus and minus buttons to
 hide/show parts of the tree.
 
-=head1 METHODS
+=head2 OPTIONS
+
+C<Tk::XMLViewer> supports all option of C<Tk::Text> and additionally
+the following:
+
+=over
+
+=item -tagcolor => $color
+
+Foreground color of tags.
+
+=item -attrkeycolor => $color
+
+Foreground color of attribute keys.
+
+=item -attrvalcolor => $color
+
+Foreground color of attribute values.
+
+=item -commentcolor => $color
+
+Foreground color of comment sections.
+
+=back
+
+The text tags C<xml_tag>, C<xml_attrkey>, C<xml_attrval>, and
+C<xml_comment> are defined for the corresponding XML elements. If you
+want to customize further you can configure the tags directly, for
+example:
+
+    $xmlviewer->tagConfigure('xml_comment', -foreground => "white",
+			     -background => "red", -font => "Helvetica 6");
+
+
+=head2 METHODS
 
 =over 4
 
@@ -529,11 +579,19 @@ a file and -text to insert an XML string.
 
 =item DumpXML
 
-Dump the contents of an Tk::Text widget into an XML string, which can
-be used as input for the XMLViewer widget. Use the static variant for
-Tk::Text widgets and the method for XMLViewer widgets.
+Dump the contents of an C<Tk::Text> widget into an XML string. This is
+meant as a alternative to the C<Tk::Text::dump> method (in fact,
+C<DumpXML> is implemented with the help of C<dump>).
+
+The output of C<DumpXML> can be used as input for the XMLViewer
+widget, which is useful in debugging C<Tk::Text> tags.
+
+Use the static variant of C<DumpXML> for C<Tk::Text> widgets and the
+method variant for C<XMLViewer> widgets.
 
     $xml_string1 = Tk::XMLViewer::DumpXML($text_widget);
+    $xmlviewer_widget->insertXML($xml_string1);
+
     $xml_string2 = $xmlviewer->DumpXML;
 
 =item SetIndent
@@ -591,7 +649,7 @@ head1 TODO
 
 =head1 AUTHOR
 
-Slaven Rezic, <eserte@cs.tu-berlin.de>
+Slaven Rezic, <slaven@rezic.de>
 
 Some additions by Jerry Geiger <jgeiger@rios.de>.
 
